package com.ai.ipu.cache.redis;

import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.locks.ReentrantLock;

import org.jsoup.helper.StringUtil;

import redis.clients.jedis.HostAndPort;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisCluster;
import redis.clients.jedis.JedisPool;
import redis.clients.jedis.JedisPoolConfig;

import com.ai.ipu.cache.config.IpuCacheConfig;
import com.ai.ipu.cache.util.IpuCacheConstant;

/**
 * redis缓存工厂类
 */
public class RedisCacheFactory {
    private static JedisPool pool;
    private static final int DEFAULT_POOL_SIZE = 8;
    private static final int DEFAULT_MAX_IDLE = 8;
    private static final int DEFAULT_MIN_IDLE = 1;
    private static final int DEFAULT_SO_TIMEOUT = 5000;
    private static final int DEFAULT_CONN_TIMEOUT = 5000;
    private static final int DEFAULT_MAX_ATTEMPTS = 3;
    protected static ReentrantLock reLock = new ReentrantLock();
    
    private RedisCacheFactory() {
    }

    /**
     * 获取redis 集群管理类
     * @param cacheName 相应的配置项的名称
     * @return redis集群管理类
     * @throws Exception
     */
    public static JedisCluster getJedisCluster(String cacheName) throws Exception{
        String clientType = IpuCacheConfig.getCacheDefaultAttr(cacheName, 
                IpuCacheConstant.Redis.CLIENT_TYPE, IpuCacheConstant.ClientType.JEDIS_CLUSTER_CLIENT);
        //异常："JedisCluster连接池中没有可用的连接，请确认缓存地址是否配置正确、缓存是否开启！"
        if(!IpuCacheConstant.ClientType.JEDIS_CLUSTER_CLIENT.equals(clientType)){
            return null;
        }else{
            return createJedisCluster(cacheName);
        }
    }

    public static JedisCluster createJedisCluster(String cacheName) throws Exception {
        List<Map<String,String>> clusterList = IpuCacheConfig.getCacheServers(cacheName);
        if (null == clusterList || clusterList.isEmpty())
            throw new IllegalArgumentException("请确认server是否配置正确");
        
        Set<HostAndPort> nodes = new HashSet<HostAndPort>();
        Map<String, String> server; 
        for (int i = 0; i < clusterList.size(); i++) {
            server = clusterList.get(i);
            nodes.add(new HostAndPort(server.get(IpuCacheConstant.Redis.IP), 
                    Integer.parseInt(server.get(IpuCacheConstant.Redis.PORT))));
        }
        
        
        JedisPoolConfig config = createPool(cacheName);
        /*设置其他参数*/
        int soTimeout   = DEFAULT_SO_TIMEOUT;
        int connTimeout = DEFAULT_CONN_TIMEOUT;
        int maxAttempts = DEFAULT_MAX_ATTEMPTS;
        String strSoTimeout = IpuCacheConfig.getCacheAttr(cacheName, IpuCacheConstant.Redis.SO_TIMEOUT);
        String strConnTimeout = IpuCacheConfig.getCacheAttr(cacheName, IpuCacheConstant.Redis.CONN_TIMEOUT);
        String strMaxAttempts = IpuCacheConfig.getCacheAttr(cacheName, IpuCacheConstant.Redis.MAX_ATTEMPS);
        if (!StringUtil.isBlank(strSoTimeout)){
            soTimeout = Integer.parseInt(strSoTimeout);
        }
        if (!StringUtil.isBlank(strConnTimeout)){
            connTimeout = Integer.parseInt(strConnTimeout);
        }
        if (!StringUtil.isBlank(strMaxAttempts)){
            maxAttempts = Integer.parseInt(strMaxAttempts);
        }
        
        JedisCluster jedisCluster;
        String auth = IpuCacheConfig.getCacheAttr(cacheName, IpuCacheConstant.Redis.AUTH);
        if (StringUtil.isBlank(auth)){
            jedisCluster = new JedisCluster(nodes, connTimeout, maxAttempts, config);
        }else{
            jedisCluster = new JedisCluster(nodes, connTimeout, soTimeout, maxAttempts, auth,  config);
        }
        return jedisCluster;
    }
    
    public static Jedis getJedis(String cacheName) throws Exception{
        String clientType = IpuCacheConfig.getCacheDefaultAttr(cacheName, 
                IpuCacheConstant.Redis.CLIENT_TYPE, IpuCacheConstant.ClientType.JEDIS_CLIENT);
        //异常："Jedis连接池中没有可用的连接，请确认缓存地址是否配置正确、缓存是否开启！"
        if(!IpuCacheConstant.ClientType.JEDIS_CLIENT.equals(clientType)){
            return null;
        }else{
        	assert ! reLock.isHeldByCurrentThread();  
            reLock.lock();
            try {
                JedisPool pool = getJedisPool(cacheName);
                return pool.getResource();
            }finally {
            	reLock.unlock(); 
            }
        }
    }
    
    public static JedisPool getJedisPool(String cacheName) throws Exception{
        if(pool==null){
            pool = createJedisPool(cacheName);
        }
        return pool;
    }

    private static JedisPool createJedisPool(String cacheName) throws Exception {
        List<Map<String,String>> clusterList = IpuCacheConfig.getCacheServers(cacheName);
        if (null == clusterList || clusterList.isEmpty())
            throw new IllegalArgumentException("请确认server是否配置正确");
        //只取第一个redis地址
        Map<String, String> server = clusterList.get(0);        
        
        JedisPoolConfig config = createPool(cacheName);
        /*设置其他参数*/
        int connTimeout = DEFAULT_CONN_TIMEOUT;
        String strConnTimeout = IpuCacheConfig.getCacheAttr(cacheName, IpuCacheConstant.Redis.CONN_TIMEOUT);
        if (!StringUtil.isBlank(strConnTimeout)){
            connTimeout = Integer.parseInt(strConnTimeout);
        }
        
        String host = server.get(IpuCacheConstant.Redis.IP);
        int port = Integer.parseInt(server.get(IpuCacheConstant.Redis.PORT));
        String auth = IpuCacheConfig.getCacheAttr(cacheName, IpuCacheConstant.Redis.AUTH);
        if (StringUtil.isBlank(auth)){
            pool = new JedisPool(config, host, port, connTimeout);
        }else{
            pool = new JedisPool(config, host, port, connTimeout, auth);
        }

        return pool;
    }
    
    private static JedisPoolConfig createPool(String cacheName) throws Exception {
        JedisPoolConfig config = new JedisPoolConfig();
        /*连接耗尽时是否阻塞, false报异常,ture阻塞直到超时, 默认true*/
        config.setBlockWhenExhausted(false);
        /*默认值设置*/
        int poolSize = DEFAULT_POOL_SIZE;
        int maxIdle  = DEFAULT_MAX_IDLE;
        int minIdle  = DEFAULT_MIN_IDLE;
        
        String strPoolSize = IpuCacheConfig.getCacheAttr(cacheName, IpuCacheConstant.Redis.POOL_SIZE);
        String strMaxIdle = IpuCacheConfig.getCacheAttr(cacheName, IpuCacheConstant.Redis.MAX_IDLE);
        String strMinIdle = IpuCacheConfig.getCacheAttr(cacheName, IpuCacheConstant.Redis.MIN_IDLE);        
        
        /*设置JedisPoolConfig参数*/
        if(!StringUtil.isBlank(strPoolSize)){
            poolSize = Integer.parseInt(strPoolSize);
        }
        if (!StringUtil.isBlank(strMaxIdle)){
            maxIdle = Integer.parseInt(strMaxIdle);
        }
        if (!StringUtil.isBlank(strMinIdle)){
            minIdle = Integer.parseInt(strMinIdle);
        }
        //最大连接数, 应用自己评估，不要超过ApsaraDB for Redis每个实例最大的连接数
        config.setMaxTotal(poolSize);
        
        //最大空闲连接数, 应用自己评估，不要超过ApsaraDB for Redis每个实例最大的连接数
        config.setMaxIdle(maxIdle);
        config.setMinIdle(minIdle);
        config.setTestOnBorrow(false);
        config.setTestOnReturn(false);
        config.setTestWhileIdle(true);
        
        return config;
    }
}
