package com.ai.ipu.cache.redis;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import com.ai.ipu.basic.log.ILogger;
import com.ai.ipu.basic.log.IpuLoggerFactory;

import junit.framework.TestCase;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPoolConfig;
import redis.clients.jedis.JedisShardInfo;
import redis.clients.jedis.Pipeline;
import redis.clients.jedis.ShardedJedis;
import redis.clients.jedis.ShardedJedisPipeline;
import redis.clients.jedis.ShardedJedisPool;
import redis.clients.jedis.Transaction;

/**
 * @author huangbo@asiainfo.com
 * @team IPU
 * @date 2018年11月6日下午4:43:44
 * @desc Jedis原始包单服务操作使用范例
 */
public class RedisTest extends TestCase {
	private static final transient ILogger LOGGER = IpuLoggerFactory.createLogger(RedisTest.class);
	private Jedis jedis;
	private ShardedJedis shardedJedis;
	private String host;
	private int port;
	
	@Override
	protected void setUp() throws Exception {
		// TODO Auto-generated method stub
		super.setUp();
		host = "47.105.160.21";
		port = 7001;
	}
	
	/**
	 * 一、普通同步调用方式
	 */
	public void testNormal() {
		jedis = new Jedis(host, port);
		jedis.auth("ipu@321");
		long start = System.currentTimeMillis();
		List<Object> results = new ArrayList<Object>();
		for (int i = 0; i < 100; i++) {
			String result = jedis.set("n" + i, "n" + i);
			results.add(result);
		}
		LOGGER.debug("results.size()==="+results.size());
		long end = System.currentTimeMillis();
		LOGGER.debug("普通同步调用方式耗时: " + ((end - start) / 1000.0) + "秒");
		jedis.disconnect();
	}
	
	/**
	 * 二、事物调用方式
	 * discard()取消事务
	 */
	public void testTrans() { 
		jedis = new Jedis(host, port);
	    jedis.auth("Ipu@321!");
	    long start = System.currentTimeMillis(); 
	    Transaction tx = jedis.multi(); 
	    for (int i = 0; i < 10000; i++) { 
	        tx.set("t" + i, "t" + i); 
	    } 
	    List<Object> results = tx.exec();
	    LOGGER.debug("results.size()==="+results.size());
	    long end = System.currentTimeMillis(); 
	    LOGGER.debug("事物调用方式耗时: " + ((end - start)/1000.0) + " seconds");
	    jedis.disconnect(); 
	} 

	
	/**
	 * 三、管道调用方式
	 */
	public void testPipe() {
		// TODO Auto-generated method stub
		jedis = new Jedis(host, port);
	    jedis.auth("Ipu@321!");
	    Pipeline pipeline = jedis.pipelined(); 
	    long start = System.currentTimeMillis(); 
	    for (int i = 0; i < 10000; i++) { 
	        pipeline.set("p" + i, "p" + i); 
	    } 
	    List<Object> results = pipeline.syncAndReturnAll(); 
	    LOGGER.debug("results.size()==="+results.size());
	    long end = System.currentTimeMillis(); 
	    LOGGER.debug("管道调用方式耗时: " + ((end - start)/1000.0) + " seconds");
	    jedis.disconnect(); 
	}
	
	/**
	 * 四、管道事务调用方式
	 */
	public void testPipeTrans() {
		// TODO Auto-generated method stub
		jedis = new Jedis(host, port);
	    jedis.auth("ipu"); 
	    long start = System.currentTimeMillis(); 
	    Pipeline pipeline = jedis.pipelined(); 
	    pipeline.multi(); 
	    for (int i = 0; i < 10000; i++) { 
	        pipeline.set("" + i, "" + i); 
	    } 
	    pipeline.exec(); 
	    List<Object> results = pipeline.syncAndReturnAll();
	    LOGGER.debug("results.size()==="+results.size());
	    long end = System.currentTimeMillis(); 
	    LOGGER.debug("管道事务调用方式耗时: " + ((end - start)/1000.0) + " seconds"); 
	    jedis.disconnect(); 
	}
	
	/**
	 * 五、普通分布式调用方式
	 * 默认hash方式存储
	 */
	public void testShardNormal() {
		// TODO Auto-generated method stub
		List<JedisShardInfo> distribut = Arrays.asList( 
	            new JedisShardInfo(host,6379), 
	            new JedisShardInfo(host,6380)); 

	    shardedJedis = new ShardedJedis(distribut);
	    List<Object> results = new ArrayList<Object>();
	    long start = System.currentTimeMillis(); 
	    for (int i = 0; i < 100000; i++) { 
	        String result = shardedJedis.set("sn" + i, "n" + i);
	        results.add(result);
	    }
	    LOGGER.debug("results.size()==="+results.size());
	    long end = System.currentTimeMillis(); 
	    LOGGER.debug("普通分布式调用方式耗时: " + ((end - start)/1000.0) + " seconds"); 

	    shardedJedis.disconnect(); 
	}
	
	/**
	 * 六、管道分布式调用方式
	 */
    public void testShardPipe() {
        // TODO Auto-generated method stub
        List<JedisShardInfo> shards = Arrays.asList(new JedisShardInfo(host, 6379), 
                new JedisShardInfo(host, 6380));

        shardedJedis = new ShardedJedis(shards);
        ShardedJedisPipeline pipeline = shardedJedis.pipelined();
        long start = System.currentTimeMillis();
        for (int i = 0; i < 100000; i++) {
            pipeline.set("sp" + i, "p" + i);
        }
        List<Object> results = pipeline.syncAndReturnAll();
        LOGGER.debug("results.size()===" + results.size());
        long end = System.currentTimeMillis();
        LOGGER.debug("Pipelined@Sharing SET: " + ((end - start) / 1000.0) + " seconds");

        shardedJedis.disconnect();
    }
	
	/**
	 * 七、分布式连接池同步调用方式
	 */
	public void testShardPool() {
		// TODO Auto-generated method stub
		List<JedisShardInfo> shards = Arrays.asList(
				new JedisShardInfo(host, 6379), 
				new JedisShardInfo(host, 6380));
		ShardedJedisPool pool = new ShardedJedisPool(new JedisPoolConfig(), shards);
		shardedJedis = pool.getResource();

		List<Object> results = new ArrayList<Object>();
		long start = System.currentTimeMillis();
		for (int i = 0; i < 100000; i++) {
			String result = shardedJedis.set("spn" + i, "n" + i);
			results.add(result);
		}
		
		LOGGER.debug("results.size()===" + results.size());
		long end = System.currentTimeMillis();
		pool.returnResource(shardedJedis);
		LOGGER.debug("Simple@Pool SET: " + ((end - start) / 1000.0) + " seconds");

		pool.destroy();
	}
	
	/**
	 * 八、分布式连接池异步调用方式
	 */
	public void testShardPipePool() {
		// TODO Auto-generated method stub
		List<JedisShardInfo> shards = Arrays.asList( 
	            new JedisShardInfo("localhost",6379), 
	            new JedisShardInfo("localhost",6380)); 

	    ShardedJedisPool pool = new ShardedJedisPool(new JedisPoolConfig(), shards); 
	    shardedJedis = pool.getResource(); 
	    ShardedJedisPipeline pipeline = shardedJedis.pipelined(); 

	    long start = System.currentTimeMillis(); 
	    for (int i = 0; i < 100000; i++) { 
	        pipeline.set("sppn" + i, "n" + i); 
	    } 
	    List<Object> results = pipeline.syncAndReturnAll();
	    LOGGER.debug("results.size()===" + results.size());
	    long end = System.currentTimeMillis(); 
	    pool.returnResource(shardedJedis); 
	    LOGGER.debug("Pipelined@Pool SET: " + ((end - start)/1000.0) + " seconds"); 
	    pool.destroy(); 
	}
	
	public void testGet() {
        jedis = new Jedis(host, port);
        long start = System.currentTimeMillis();
        jedis.keys("*");
        String result = jedis.get("msg");
        LOGGER.debug("返回结果："+result);
        LOGGER.debug("普通同步调用方式耗时: " + (System.currentTimeMillis() - start) + "ms");
        jedis.disconnect();
    }
}
